@@ -1,7 +1,7 @@
 // see `mlogit-ex.cpp` for an example with more comments
 
 // we will use openMP to perform the comptutation in parallel
-// [[Rcpp::plugins(openmp)]]
+// [[Rcpp::plugins(openmp, cpp11)]]
 
 // we use RcppArmadillo to simplify the code
 // [[Rcpp::depends(RcppArmadillo)]]
@@ -67,7 +67,7 @@ public:
   }
 
   double grad
-  (double const * __restrict__ point, double * __restrict__ gr) const {
+  (double const * point, double * gr) const {
     arma::vec const x_glob = vec_no_cp(point           , n_global),
                     x_priv = vec_no_cp(point + n_global, mu_cluster.n_elem),
                      delta = x_priv - Psi * x_glob(g_idx) - mu_cluster;
@@ -125,7 +125,7 @@ List optim_poly
    unsigned const n_threads, double const c1,
    double const c2, bool const use_bfgs = true, int const trace = 0L,
    double const cg_tol = .5, bool const strong_wolfe = true,
-   size_t const max_cg = 0L){
+   size_t const max_cg = 0L, int const pre_method = 1L){
   XPtr<poly_optim> optim(ptr);
 
   // check that we pass a parameter value of the right length
@@ -135,7 +135,8 @@ List optim_poly
   NumericVector par = clone(val);
   optim->set_n_threads(n_threads);
   auto res = optim->optim(&par[0], rel_eps, max_it, c1, c2,
-                          use_bfgs, trace, cg_tol, strong_wolfe, max_cg);
+                          use_bfgs, trace, cg_tol, strong_wolfe, max_cg,
+                          static_cast<PSQN::precondition>(pre_method));
   NumericVector counts = NumericVector::create(
     res.n_eval, res.n_grad,  res.n_cg);
   counts.names() = CharacterVector::create("function", "gradient", "n_cg");
